@@ -1,8 +1,25 @@
 # -------------------------------------------------------------------------------------
 # MKDoc::Handler::Initialize
 # -------------------------------------------------------------------------------------
-# Author: Jean-Michel Hiver <jhiver@mkdoc.com>.
+# Author: Jean-Michel Hiver.
 # Copyright: (c) MKDoc Holdings Ltd, 2001
+#
+# This file is part of MKDoc. 
+# 
+# MKDoc is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+# 
+# MKDoc is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with MKDoc; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
 # Description: Initializes MKDoc framework for subsequent handlers.
 # -------------------------------------------------------------------------------------
 package MKDoc::Handler::Initialize;
@@ -13,6 +30,7 @@ use MKDoc::Config;
 use Carp;
 use lib::sql::DBH;
 use lib::sql::Table;
+use MKDoc::CGI;
 
 use CGI qw(-compile :all);
 
@@ -29,7 +47,16 @@ sub handler
     + lib::sql::DBH->disconnect;
     
     clean_globals();
-    
+
+    $ENV{USE_CGI} and do {
+        open FP, "<$ENV{USE_CGI}" or die "Cannot read-open $ENV{USE_CGI}";
+        my $data = join '', <FP>;
+        close FP;
+
+        eval $data;
+        die $@ if ($@);
+    };
+ 
     $Petal::DECODE_CHARSET = 'utf8';
     $Petal::ENCODE_CHARSET = 'utf8';    
     $Petal::BASE_DIR = undef;
