@@ -177,6 +177,7 @@ sub links
 	foreach my $link_component (@links_components)
 	{
 	    push @links, {
+               class => "mkdoc-link",
 		href => $link_component->url,
 		desc => $link_component->description,
 		expr => $link_component->title,
@@ -184,9 +185,10 @@ sub links
 	}
 	
 	# plus we want to hyperlink all the children
-	foreach my $child ($document->children)
+	foreach my $child ($document->children_showable)
 	{
 	    push @links, {
+               class => "mkdoc-name-". $child->name,
 		href => $child->uri,
 		desc => $child->description,
 		expr => $child->title,
