@@ -436,6 +436,9 @@ sub delete
 {
     my $self = shift;    
     $self->is_root() and return;
+
+    # call delete handler
+    $self->on_delete();
     
     $_->delete() for ($self->children());
     $self->_delete_redirect (@_);
@@ -2591,5 +2594,33 @@ sub now_time_iso
     return $time;
 }
 
+##
+# $self->on_save;
+#
+# Handler which gets called just before saving the document.  Needed
+# to clean up index tables, like Document_TimeRange.
+##
+sub on_save {
+    my $self = shift;
+    my $id   = $self->id();
+
+    # new documents won't need pre-save cleanup
+    return unless $id;
+
+    my $dbh = lib::sql::DBH->get();
+    $dbh->do('DELETE FROM Document_TimeRange WHERE Document_ID = ?',
+             undef, $id);
+}
+
+##
+# $self->on_delete;
+#
+# Handler which gets called just before deleting the document.  Needed
+# to clean up index tables, like Document_TimeRange.
+##
+sub on_delete {
+    my $self = shift;
+    $self->on_save();
+}
 
 1;
