@@ -67,7 +67,7 @@ sub run
     
     print <<EOF;
 Status: $status
-Content-Type: text/plain
+Content-Type: text/html; charset=UTF-8
 
 $@
 EOF
