@@ -8,6 +8,7 @@ use flo::Record::Document;
 use Encode;
 use base qw /flo::Plugin/;
 use Petal::Mail;
+use Carp qw(croak);
 
 sub template_path { '/newsletter' }
 
@@ -43,16 +44,73 @@ sub send
     return 1; 
 }
 
+# returns true if there are newest documents to include in the newsletter
+sub has_newest {
+    my $self = shift;
+    my $res = $self->results("newest");
+    return 1 if @$res;
+    return 0;
+}
 
-sub results
-{
+# returns true if there are upcoming events to include in the newsletter
+sub has_upcoming {
+    my $self = shift;
+    my $res = $self->results("upcoming");
+    return 1 if @$res;
+    return 0;
+}
+
+# return ref to array of hashes containing newsletter results.  Takes
+# a single parameter specifying the mode, either "newest" or
+# "upcoming".  This method cache the results for each mode.
+sub results {
     my $self = shift;    
+    my $mode = shift || croak("Missing required mode parameter");
+
+    # return cached data if available
+    return $self->{"_cache_$mode"} if $self->{"_cache_$mode"};
+
     my $user_login = $self->user()->login();
-    my $date_since = $self->compute_date_since();
-    
+
+    # concoct SQL needed for upcoming or newest mode
+    my ($extra_from, $extra_where, $extra_select, @extra_params,
+        $order_by, $group_by);
+        
+    if ($mode eq 'upcoming') {
+        # setup for selecting by timerange
+        $order_by     = "Document_TimeRange.FromDate ASC";
+        $group_by     = "Document_TimeRange.ID";
+        $extra_from   = ", Document_TimeRange";
+        $extra_select = ", Document_TimeRange.ID as Document_TimeRange_ID";
+
+        # select documents with timeranges dates from now to the end
+        # of the range
+        $extra_where  = <<END;
+          AND Document.ID = Document_TimeRange.Document_ID
+          AND ((Document_TimeRange.FromDate >= ? AND 
+                Document_TimeRange.FromDate <= ?)
+               OR 
+               (Document_TimeRange.ToDate >= ? AND 
+                Document_TimeRange.ToDate <= ?))
+END
+        @extra_params = ($self->compute_date_now(), 
+                         $self->compute_date_forward()) x 2;
+    } else {
+        # select documents created in the chosen period
+        $order_by     = "Date_Created DESC";
+        $group_by     = "Document.ID";
+        $extra_where  = "AND Document.Date_Created > ?";
+        ($extra_from, $extra_select) = ("") x 2;
+        @extra_params = $self->compute_date_since();
+    }
+
     my $sql  = <<SQL;
-SELECT Document.ID AS ID, SUM(Preference_Audience.Value) AS Pref_Score
+SELECT Document.ID AS ID, SUM(Preference_Audience.Value) AS Pref_Score,
+       Creator.First_Name as Creator_First_Name,
+       Creator.Family_Name as Creator_Family_Name
+       $extra_select
 FROM   Document, Document_Audience, Audience, Preference_Audience, Editor, Editor as Creator, Preference_Language
+       $extra_from
 WHERE
         -- join the tables together
         (
@@ -70,68 +128,92 @@ AND
 AND
         -- remove languages which are not wanted
         Preference_Language.Value = 1
-AND
-        -- get only documents which are recent enough
-       Document.Date_Created > ?
+$extra_where
 
-GROUP BY (Document.ID)
-ORDER BY Document.Full_Path
+
+GROUP BY $group_by
+ORDER BY $order_by
 SQL
 
     my $dbh = lib::sql::DBH->get();
     my $sth = $dbh->prepare_cached ($sql);
-    $sth->execute ($user_login, $date_since);
+    $sth->execute ($user_login, @extra_params);
     
-    my @res = ();
+    my @res;
     my $doc_t = flo::Standard::table ('Document');
-    while (my $h = $sth->fetchrow_hashref)
-    {
+    ROW: while (my $h = $sth->fetchrow_hashref) {
         next unless ($h->{Pref_Score} > 0);
 	my $doc = $doc_t->get ( $h->{ID} );
 	next unless ($doc->is_showable());
 	
 	$Text::Wrap::columns = 72;
 	$Text::Wrap::columns = 72;
-	$h->{Full_Path} =~ s/^\//$::PUBLIC_DOMAIN/;
-	$h->{Date_Created} =~ s/\s+.*//;
-	$h->{Description} = join '', fill ('    ', '    ', $h->{Description});
-	push @res, $h;
+
+        # assemble data for template from the document object and the
+        # query results
+        my %row;
+        $row{doc}           = $doc;
+        ($row{Full_Path}    = $doc->{Full_Path}) =~ s/^\//$::PUBLIC_DOMAIN/;
+        ($row{Date_Created} = $doc->date_created) =~ s/\s+.*//;
+        $row{Description}   = join '',fill ('    ', '    ', $doc->description);
+        $row{Title}         = $doc->title;
+        $row{Creator_First_Name}  = $h->{Creator_First_Name};
+        $row{Creator_Family_Name} = $h->{Creator_Family_Name};
+
+        # deal with timerange data for upcoming events list
+        if ($mode eq 'upcoming') {
+            # find this timerange in the list of components for this doc
+            my $timerange_id = $h->{Document_TimeRange_ID};
+            my ($timerange) = 
+              grep { $_->isa('flo::editor::TimeRange') and
+                     $_->Document_TimeRange_ID == $timerange_id }
+                $doc->components;
+            # ignore hits with no matching timerange.
+            next ROW unless $timerange;
+
+            $row{TimeRange_Title}    = $timerange->title;
+            $row{TimeRange_FromDate} = $timerange->from_datetime;
+            $row{TimeRange_ToDate}   = $timerange->to_datetime;
+        }
+
+	push @res, \%row;
     }
     
-    return \@res;
+    # save for future requests
+    return $self->{"_cache_$mode"} = \@res;
 }
 
+# translate a time to day-month-year for use with MySQL
+sub _time2date {
+    my $self = shift;
+    my ($mday, $mon, $year) = (localtime(shift))[3,4,5];
+    $mon += 1;
+    $year += 1900;
+    return "$year-$mon-$mday";
+}
+
+sub compute_date_now { shift->_time2date(time()) }
 
 package MKDoc::Misc::Newsletter::Daily;
 use strict;
 use warnings;
 our @ISA = qw /MKDoc::Misc::Newsletter/;
 
-sub compute_date_since
-{
-    my $self = shift;
-    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time - 24 * 3600);
-    $mon += 1;
-    $year += 1900;
-    return "$year-$mon-$mday";
-}
+sub compute_date_since { shift->_time2date(time - 24 * 3600) }
+
+sub compute_date_forward { shift->_time2date(time + 24 * 3600) . " 23:59:59" }
 
 sub is_daily { 1 }
 
-
 package MKDoc::Misc::Newsletter::Weekly;
 use strict;
 use warnings;
 our @ISA = qw /MKDoc::Misc::Newsletter/;
 
-sub compute_date_since
-{
-    my $self = shift;
-    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time - 7 * 24 * 3600);
-    $mon += 1;
-    $year += 1900;
-    return "$year-$mon-$mday";
-}
+sub compute_date_since { shift->_time2date(time - 7 * 24 * 3600) }
+
+sub compute_date_forward { shift->_time2date(time + 7 * 24 * 3600) . 
+                             " 23:59:59" }
 
 sub is_weekly { 1 }
 
@@ -141,10 +223,9 @@ use strict;
 use warnings;
 our @ISA = qw /MKDoc::Misc::Newsletter/;
 
-sub compute_date_since
-{
+sub compute_date_since {
     my $self = shift;
-    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time);
+    my ($mday, $mon, $year) = (localtime(time))[3,4,5];
     $year += 1900;
     ($mon == 0) and do {
 	$year--;
@@ -154,6 +235,19 @@ sub compute_date_since
     return "$year-$mon-$mday";
 }
 
+sub compute_date_forward {
+    my $self = shift;
+    my ($mday, $mon, $year) = (localtime(time))[3,4,5];
+    $year += 1900;
+    $mon  += 2;
+    ($mon == 13) and do {
+	$year++;
+	$mon = 1;
+    };
+    
+    return "$year-$mon-$mday 23:59:59";
+}
+
 sub is_monthly { 1 }
 
 
@@ -206,8 +300,8 @@ sub instantiate_newsletter
     ($periodicity eq WEEKLY)  and $news = new MKDoc::Misc::Newsletter::Weekly (%args);
     ($periodicity eq MONTHLY) and $news = new MKDoc::Misc::Newsletter::Monthly (%args);
     
-    my $res = $news->results();
-    @{$res} or return;
+    # don't send a newsletter if there's no news
+    return unless $news->has_upcoming or $news->has_newest;
     return $news;
 }
 
